import speech_recognition as sr
import os
from gtts import gTTS
from playsound import playsound
import sys
import configparser
import random
from lazyme.string import color_print as cprint

try:
    import pyaudio
except Exception as e:
    try:
        os.system("pipwin install pyaudio")
    except Exception as e:
        try:
            os.system("pip install pyaudio")
        except Exception as e:
            print("Exception occur ", e)
            print("Install pyaudio manually")
        import pyaudio

# import custom features
try:
    import features.weather.weather as wea
    import features.website_open.website_open
    import features.send_mail.send_mail
    import features.date_time.date_time
    import features.launch_app.launch_app
    import features.news.news as nw
    import features.tell_me_about.tell_me_about as tma
    import features.face_recognition.dataset_create as dc
    import features.face_recognition.train as train
    import features.face_recognition.predict as predict
    import features.face_recognition.face_reco as fr
    import features.show_me_my_images.show_me_my_images as my_photos
    import features.setup.setup as setup_assistant
    import features.google_photos.google_photos as gp
    import features.joke.joke
    import features.hot_word_detection.hot_word_detection as wake_word
    import features.mic_input_ai.mic_input_ai.SpeechRecognition as SpeechRecognitionAI
    import features.jarvisai_api.jarvisai_api.JarvisAIAPI
    import features.detection.detection.Detections
except Exception as e:
    from JarvisAI.features.weather import weather as wea
    from JarvisAI.features.website_open import website_open
    from JarvisAI.features.send_mail import send_mail
    from JarvisAI.features.date_time import date_time
    from JarvisAI.features.launch_app import launch_app
    from JarvisAI.features.news import news as nw
    from JarvisAI.features.tell_me_about import tell_me_about as tma
    from JarvisAI.features.face_recognition import dataset_create as dc
    from JarvisAI.features.face_recognition import train as train
    from JarvisAI.features.face_recognition import predict as predict
    from JarvisAI.features.face_recognition import face_reco as fr
    from JarvisAI.features.show_me_my_images import show_me_my_images as my_photos
    from JarvisAI.features.setup import setup as setup_assistant
    from JarvisAI.features.google_photos import google_photos as gp
    from JarvisAI.features.joke import joke
    from JarvisAI.features.hot_word_detection import hot_word_detection as wake_word
    from JarvisAI.features.mic_input_ai.mic_input_ai import SpeechRecognition as SpeechRecognitionAI
    from JarvisAI.features.jarvisai_api.jarvisai_api import JarvisAIAPI
    from JarvisAI.features.detection.detection import Detections


class JarvisAssistant:
    def __init__(self, sync=True, token=None):
        if token is None or not sync:
            print("\n")
            cprint("Set 'obj = JarvisAI.JarvisAssistant(sync=False, token='xyz')' if you do not want to use API",
                   color='green')
            cprint("Obtain your token from: http://jarvis-ai-api.herokuapp.com/", color='green')
            cprint("Its free to use. Even API services is also free. \n"
                   "Well, JarvisAI need support of your to keep this project and it's API alive.\n"
                   "So, your contribution/donation will be appreciated. \n"
                   , color='blue')
            cprint("DONATE: https://www.buymeacoffee.com/dipeshpal", color='red')
        self.sync = sync
        self.token = token
        self.speech_recognition_ai = SpeechRecognitionAI()
        self.jarvisai_api = JarvisAIAPI()
        self.obj_detection = Detections()

    def setup(self):
        """
        Method to define configuration related to assistant
        :return: Bool
            True if setup done
            False if setup cancel or interrupt
        """
        obj_setup = setup_assistant.Setup()
        response = obj_setup.setup_assistant()
        del obj_setup
        return response

    def hot_word_detect(self, lang='en'):
        """
        Hot word (wake word / background listen) detection
        :param lang: str
            default 'en'
        :return: Bool, str
            status, command
        """
        return wake_word.hot_word_detection(lang=lang)

    def mic_input(self, lang='en'):
        """
        Fetch input from mic
        Note: mic_input usages Google's Speech Recognition
        Limitation: After multiple hits API may not work so use mic_input_ai() instead of mic_input
        :param lang: str
            default 'en'
        :return: str/Bool
            user's voice input as text if true/ false if fail
        """
        if self.sync == False:
            config = configparser.ConfigParser()
            config.read('config/config.ini')
            user_name = config['default']['user_name']
        else:
            if self.token is None:
                cprint("TOKEN NOT FOUND",
                       color='yellow')
                cprint("Set 'obj = JarvisAI.JarvisAssistant(sync=true, token='xyz')' if you want to use this API",
                       color='red')
                cprint("Obtain your token from: http://jarvis-ai-api.herokuapp.com/", color='green')
                print("Currently using local config from 'config/config.ini' . . .")
                config = configparser.ConfigParser()
                config.read('config/config.ini')
                user_name = config['default']['user_name']
            else:
                status, res = self.jarvisai_api.get_user_data(self.token)
                user_name = res['data']['name_']

        try:
            r = sr.Recognizer()
            with sr.Microphone() as source:
                greeting = random.choice(["Hello", "Hi"])
                msg = greeting + ' ' + user_name + ',' + ' How may I help you?'
                print(msg)
                self.text2speech(msg)
                r.pause_threshold = 1
                r.adjust_for_ambient_noise(source, duration=1)
                audio = r.listen(source)
            try:
                command = r.recognize_google(audio, language=lang).lower()
                print('You said: ' + command + '\n')
            except sr.UnknownValueError:
                print('....')
                command = self.mic_input()
            return command
        except Exception as e:
            print(e)
            return False

    def mic_input_ai(self, record_seconds=5, debug=False):
        """
        Fetch input from mic and recognition using Transformers
        Note: This will download pretrained ML model for the first time only
        :param record_seconds: int
            Default 5
        :param debug: bool
            Print recording status if True
        :return: str
            User's voice input as text
        """
        transcription = self.speech_recognition_ai.start_speech_recognition(record_seconds=record_seconds, debug=debug)
        return transcription

    def text2speech(self, text, lang='en'):
        """
        Convert any text to speech
        :param text: str
            text (String)
        :param lang: str
            default 'en'
        :return: Bool
            True / False (Play sound if True otherwise write exception to log and return False)
        """
        try:
            myobj = gTTS(text=text, lang=lang, slow=False)
            myobj.save("tmp.mp3")
            playsound("tmp.mp3")
            os.remove("tmp.mp3")
            return True
        except Exception as e:
            mytext = "Sorry I couldn't understand, or not implemented to handle this input"
            print(mytext)
            myobj = gTTS(text=mytext, lang=lang, slow=False)
            myobj.save("tmp.mp3")
            playsound("tmp.mp3")
            os.remove("tmp.mp3")
            print(e)
            return False

    def shutdown(self):
        """
        Shutdown the Jarvis API, exit from program
        :return: None/bool
            if no error then exit from program, False if Fail
        """
        try:
            self.text2speech('Good bye. Have a nice day')
            sys.exit()
        except Exception as e:
            print(e)
            return False

    def website_opener(self, domain):
        """
        This will open website according to domain
        :param domain: str
            any domain, example "youtube.com"
        :return: Bool
            True if success, False if fail
        """
        return website_open.website_opener(domain)

    def send_mail(self, sender_email=None, sender_password=None, receiver_email=None, msg="Hello"):
        """
        This function will send mail to user according to input params
        Currently on =ly support Gmail
        :param sender_email: str
            Email id of sender
        :param sender_password: str
            Password
        :param receiver_email: str
            Email id of receiver
        :param msg: str
            Message or mail you want to send
        :return: Bool
            True if success, False if fail
        """
        return send_mail(sender_email, sender_password, receiver_email, msg)

    def tell_me_date(self):
        """
        Just return date as string
        :return: str/Bool
            date if success, False if fail
        """
        return date_time.date()

    def tell_me_time(self):
        """
        This function will return time
        :return: str/Bool
            Time if success, False if fail
        """
        return date_time.time()

    def launch_any_app(self, path_of_app='C:\Program Files (x86)\Google\Chrome\Application\chrome.exe'):
        """
        Launch any windows application according to application path
        :param path_of_app: str
            path of exe
        :return: Bool
            True if success and open the application, False if fail
        """
        return launch_app.launch_app(path_of_app)

    def weather(self, city='Indore'):
        """
        Return weather
        :param city: str
            Any city of this world
        :return: str/bool
            weather info as string if True, or False
        """
        try:
            res = wea.weather_app(city)
        except Exception as e:
            print(e)
            res = False
        return res

    def news(self):
        """
        Fetch top news of the day from news.google.com/news/rss
        :return: list/bool
            news list of string if True, False if fail
        """
        return nw.news()

    def tell_me(self, topic='tell me about Taj Mahal'):
        """
        Tells about anything from wikipedia
        :param topic: str
            any string is valid options
        :return: list/bool
            First 500 character from wikipedia if True, False if fail
        """
        return tma.tell_me_about(topic)

    def datasetcreate(self, dataset_path='datasets', class_name='Demo',
                      haarcascade_path='haarcascade/haarcascade_frontalface_default.xml',
                      eyecascade_path='haarcascade/haarcascade_eye.xml', eye_detect=False,
                      save_face_only=True, no_of_samples=100,
                      width=128, height=128, color_mode=False):
        """
        Dataset Create by face detection
        :param dataset_path: str (example: 'folder_of_dataset')
        :param class_name: str (example: 'folder_of_dataset')
        :param haarcascade_path: str (example: 'haarcascade_frontalface_default.xml)
        :param eyecascade_path: str (example: 'haarcascade_eye.xml)
        :param eye_detect: bool (example:True)
        :param save_face_only: bool (example:True)
        :param no_of_samples: int (example: 100)
        :param width: int (example: 128)
        :param height: int (example: 128)
        :param color_mode: bool (example:False)
        :return: None
        """

        obj = fr.DatasetCreate(dataset_path=dataset_path, class_name=class_name,
                               haarcascade_path=haarcascade_path,
                               eyecascade_path=eyecascade_path, eye_detect=eye_detect,
                               save_face_only=save_face_only, no_of_samples=no_of_samples,
                               width=width, height=height, color_mode=color_mode)
        obj.datasetcreate()

    def face_recognition_train(self, data_dir='datasets', batch_size=32, img_height=128, img_width=128, epochs=10,
                               model_path='model', pretrained=None, base_model_trainable=False):
        """
        Train TF Keras model according to dataset path
        :param data_dir: str (example: 'folder_of_dataset')
        :param batch_size: int (example:32)
        :param img_height: int (example:128)
        :param img_width: int (example:128)
        :param epochs: int (example:10)
        :param model_path: str (example: 'model')
        :param pretrained: str (example: None, 'VGG16', 'ResNet50', 'InceptionV3')
        :param base_model_trainable: bool (example: False (Enable if you want to train the pretrained model's layer))
        :return: None
        """
        obj = fr.FaceRecognizerTrain(data_dir=data_dir, batch_size=batch_size, img_height=img_height,
                                     img_width=img_width, epochs=epochs, model_path=model_path, pretrained=pretrained,
                                     base_model_trainable=base_model_trainable)
        obj.train()

    def predict_faces(self, class_name=None, img_height=128, img_width=128,
                      haarcascade_path='haarcascade/haarcascade_frontalface_default.xml',
                      eyecascade_path='haarcascade/haarcascade_eye.xml', model_path='model',
                      color_mode=False):
        """
        Predict Face
        :param class_name: Type-List (example: ['class1', 'class2'] )
        :param img_height: int (example:128)
        :param img_width: int (example:128)
        :param haarcascade_path: str (example: 'haarcascade_frontalface_default.xml)
        :param eyecascade_path: str (example: 'haarcascade_eye.xml)
        :param model_path: str (example: 'model')
        :param color_mode: bool (example: False)
        :return: None
        """
        obj = fr.Predict(class_name=class_name, img_height=img_height, img_width=img_width,
                         haarcascade_path=haarcascade_path,
                         eyecascade_path=eyecascade_path, model_path=model_path,
                         color_mode=color_mode)
        obj.predictfaces()

    def show_me_my_images(self):
        """
        This function will show images from local directory. Make sure to run setup() first to setup local directory.
        :return: Bool
        """
        return my_photos.show_me_my_images()

    def show_google_photos(self):
        """
        This function will open "https://photos.google.com/" in your browser.
        :return: Bool
        """
        return gp.google_photos()

    def tell_me_joke(self, language='en', category='neutral'):
        """
        Function to tell a joke
        Read https://pyjok.es/api/ for more details
        :param language: str
            default "en"
        :param category: str
            default "neutral"
        :return: str
            "Joke:
        """
        return joke.tell_me_joke(lang=language, cat=category)

    def get_user_data(self):
        """
        Function to fetch user data
        More info: https://jarvis-ai-api.herokuapp.com/api_docs/
        :return: status, response
        """
        if self.token is None:
            cprint("TOKEN NOT FOUND",
                   color='yellow')
            cprint("Set 'obj = JarvisAI.JarvisAssistant(sync=true, token='xyz')' if you want to use this API",
                   color='red')
            cprint("Obtain your token from: http://jarvis-ai-api.herokuapp.com/", color='green')

        status, res = self.jarvisai_api.get_user_data(self.token)
        return status, res

    def set_user_data(self):
        """
        Function to set user data
        :return: None
        """
        self.jarvisai_api.set_user_data()

    def jarvisai_configure_hand_detector(self, camera=0, detectionCon=0.7, maxHands=2, cam_display=True, cam_height=480,
                                         cam_width=888):
        """
        This function is for hand detection configuration.
        Before using 'jarvisai_detect_hands' you need to call this function to setup things.
        @param camera: int (example:0)
            Your device camera number
        @param detectionCon: int (example:0.5)
            The accuracy of hand detection
        @param maxHands: int (example:2)
            Maximum number of hands you want to detect in single frame
        @param cam_display: Bool (example:True)
            Set 'False' if you do not want to display camera window
        @param cam_height: int (Example: 480)
            Resolution of camera window height
        @param cam_width: int (Example: 888)
            Resolution of camera window width
        """
        self.obj_detection.configure_hand_detector(camera, detectionCon, maxHands, cam_display, cam_height, cam_width)

    def jarvisai_detect_hands(self, message=""):
        """
        This function detect the hands and return number of fingers with hand type.
        @param message: str (example: Hello!)
        @return:
            fingers: list (Number of fingers up),
            hand_type: string ('Left' or 'Right' Hand),
            cv2: cv2 object for advance usages,
            img: image array from your camera for advance usages,
            cap: The cap is from VideoCapture object ('cap = cv2.VideoCapture(camera) ) for advance usages.'
            For ADVANCE USAGES REFER: https://docs.opencv.org/master/index.html
        """
        fingers, hand_type, cv2, img, cap = self.obj_detection.detect_hands(message)
        return fingers, hand_type, cv2, img, cap


if __name__ == '__main__':
    obj = JarvisAssistant()
    # obj.jarvisai_detect_hands()
    # print(obj.mic_input_ai())
    # print(obj.text2speech_male())
    # res = obj.tell_me_joke()
    # print(res)
    # obj.text2speech("hello")
    # res = obj.website_opener("facebook.com")
    # res = obj.send_mail()
    # res = obj.launch_app("edge")
    # res = obj.weather("mumbai")
    # res = obj.news()
    # res = obj.tell_me()
    # res = obj.tell_me_time()
    # res = obj.tell_me_date()
    # res = obj.shutdown()
    # obj.datasetcreate()



# Voice Recognition using https://huggingface.co/facebook/wav2vec2-base-960h
import pyaudio
import wave
import os
import librosa
from transformers import Wav2Vec2ForCTC, Wav2Vec2Tokenizer
from lazyme.string import color_print as cprint

try:
    import torch
except ImportError:
    cprint("No Pytorch Found, Please install if manually from below link", color='yellow', underline=True, bold=True)
    cprint("https://pytorch.org/get-started/locally/", color='blue', underline=True, bold=True)


class MicRecord:
    def __init__(self):
        self.FORMAT = pyaudio.paInt16
        self.CHANNELS = 2
        self.RATE = 44100
        self.CHUNK = 1024
        self.WAVE_OUTPUT_FILENAME = "temp_record.wav"
        if os.path.exists(self.WAVE_OUTPUT_FILENAME):
            os.remove(self.WAVE_OUTPUT_FILENAME)

    def start_recording(self, record_seconds=5, debug=False):
        audio = pyaudio.PyAudio()
        # start Recording
        stream = audio.open(format=self.FORMAT, channels=self.CHANNELS,
                                 rate=self.RATE, input=True,
                                 frames_per_buffer=self.CHUNK)
        print(f"Listening for {record_seconds} Seconds")
        if debug:
            print("recording...")
        frames = []

        for i in range(0, int(self.RATE / self.CHUNK * record_seconds)):
            data = stream.read(self.CHUNK)
            frames.append(data)
        if debug:
            print("finished recording")

        # stop Recording
        stream.stop_stream()
        stream.close()
        audio.terminate()
        waveFile = wave.open(self.WAVE_OUTPUT_FILENAME, 'wb')
        waveFile.setnchannels(self.CHANNELS)
        waveFile.setsampwidth(audio.get_sample_size(self.FORMAT))
        waveFile.setframerate(self.RATE)
        waveFile.writeframes(b''.join(frames))
        waveFile.close()


class SpeechRecognition(MicRecord):
    def __init__(self):
        super().__init__()
        self.tokenizer = Wav2Vec2Tokenizer.from_pretrained("facebook/wav2vec2-base-960h")
        self.model = Wav2Vec2ForCTC.from_pretrained("facebook/wav2vec2-base-960h")

    def start_speech_recognition(self, record_seconds=5, debug=False):
        self.start_recording(record_seconds=record_seconds, debug=debug)
        print("Recognizing...")
        input_audio, _ = librosa.load(self.WAVE_OUTPUT_FILENAME, sr=16000)
        input_values = self.tokenizer(input_audio, return_tensors="pt").input_values
        logits = self.model(input_values).logits
        predicted_ids = torch.argmax(logits, dim=-1)
        transcription = self.tokenizer.batch_decode(predicted_ids)[0]
        os.remove(self.WAVE_OUTPUT_FILENAME)
        return transcription.lower()


if __name__ == '__main__':
    obj = SpeechRecognition()
    transcription = obj.start_speech_recognition()
    print(transcription)



import speech_recognition as sr
import configparser
import re


def hot_word_detection(lang='en'):
    """
    Hot word (wake word / background listen) detection
    :param lang: str
        default 'en'
    :return: Bool, str
        status, command
    """
    config = configparser.ConfigParser()
    config.read('config/config.ini')
    bot_name = config['default']['bot_name']
    try:
        r = sr.Recognizer()
        with sr.Microphone() as source:
            r.pause_threshold = 1
            r.adjust_for_ambient_noise(source, duration=1)
            audio = r.listen(source)
            command = r.recognize_google(audio, language=lang).lower()
            if re.search(bot_name, command):
                return True, command
    except Exception as e:
        print(e)
        return False, None


if __name__ == '__main__':
    hot_word_detection()



import os
import configparser


class Setup:
    def start_sconfig(self, config):
        default = 'default'
        config.add_section('default')

        bot_name = str(input("Enter Bot name (default=jarvis): ") or "jarvis")
        user_name = str(input("Enter User name (default=dipesh): ") or "Dipesh")
        photos_dir = str(input("Enter Photos Directories separated by comma ',' (default=None): ") or "None")

        config[default]['bot_name'] = bot_name
        config[default]['user_name'] = user_name
        config[default]['photos'] = photos_dir

        with open('config/config.ini', 'w') as configfile:
            config.write(configfile)

        print("Setup Done")
        return True

    def setup_assistant(self):
        if not os.path.exists('config'):
            os.makedirs('config')
        config = configparser.ConfigParser()

        if os.path.exists('config'):
            sure = str(input("Your current configuration will be deleted, are you sure (y/n): ") or "n")
            if sure == "n":
                print("No changes done")
                return False
            else:
                return self.start_sconfig(config)
        else:
            return self.start_sconfig(config)



import cvzone
import cv2


class Detections:
    def __init__(self):
        self.cap = cv2.VideoCapture(0)
        self.detectionCon = 0.5
        self.maxHands = 2
        self.cam_display = True
        self.detector = cvzone.HandDetector(detectionCon=self.detectionCon, maxHands=self.maxHands)

    def configure_hand_detector(self, camera=0, detectionCon=0.7, maxHands=2, cam_display=True, cam_height=480, cam_width=888):
        self.cap = cv2.VideoCapture(camera)
        self.cap.set(3, cam_width)
        self.cap.set(4, cam_height)
        self.detectionCon = detectionCon
        self.maxHands = maxHands
        self.cam_display = cam_display
        self.detector = cvzone.HandDetector(detectionCon=self.detectionCon, maxHands=self.maxHands)
        # self.detector_face = cvzone.FaceMeshDetector(maxFaces=2)

    def detect_hands(self, message=""):
        success, img = self.cap.read()
        # img, faces = self.detector_face.findFaceMesh(img)
        img = self.detector.findHands(img)
        lmlist, bbox = self.detector.findPosition(img)
        fingers = None
        myHandType = None
        img = cv2.flip(img, 1)
        if lmlist:
            fingers = self.detector.fingersUp()
            totalFingers = fingers.count(1)
            myHandType = self.detector.handType()
            cv2.putText(img, f'Hand:{myHandType} | Fingers:{totalFingers}', (50, 50),
                        cv2.FONT_HERSHEY_PLAIN, 2, (0, 255, 0), 2)
            cv2.putText(img, f'{message}', (50, 100),
                        cv2.FONT_HERSHEY_PLAIN, 2, (0, 255, 0), 2)
        if self.cam_display:
            cv2.imshow('img', img)
            cv2.waitKey(1)
        return fingers, myHandType, cv2, img, self.cap


if __name__ == '__main__':
    obj = Detections()
    obj.configure_hand_detector(maxHands=2)

    import concurrent.futures

    with concurrent.futures.ThreadPoolExecutor() as executor:
        while True:
            future = executor.submit(obj.detect_hands)
            fingers, hand_type, cv2, img = future.result()
            print(fingers, hand_type)

        # fingers, hand_type, cv2, img = obj.detect_hands()
        # print(fingers, hand_type)
        # cv2.putText(img, f'{"message"}', (50, 200),
        #                 cv2.FONT_HERSHEY_PLAIN, 2, (0, 255, 0), 2)
        # cv2.imshow("img", img)
        # cv2.waitKey(1)



import wikipedia
import re


def tell_me_about(topic):
    try:
        ny = wikipedia.page(topic)
        res = str(ny.content[:500].encode('utf-8'))
        res = re.sub('[^a-zA-Z.\d\s]', '', res)[1:]
        return res
    except Exception as e:
        print(e)
        return False


import matplotlib.pyplot as plt
import os
import warnings

warnings.simplefilter("ignore")
import pathlib
import glob
import tensorflow as tf

from tensorflow import keras
from tensorflow.keras import layers
from tensorflow.keras.models import Sequential


class Classifier:
    def __init__(self, data_dir='datasets', batch_size=32, img_height=128, img_width=128, epochs=10,
                 model_path='model'):
        self.data_dir = pathlib.Path(data_dir)
        self.image_count = len(list(glob.glob(data_dir + '/*/*.png')))
        self.batch_size = batch_size
        self.img_height = img_height
        self.img_width = img_width
        self.num_classes = len(os.listdir(pathlib.Path(data_dir)))
        self.class_names = None
        self.train_ds = None
        self.val_ds = None
        self.epochs = epochs
        self.data_augmentation = None
        self.model_path = model_path

    def dataloader(self):
        train_ds = tf.keras.preprocessing.image_dataset_from_directory(
            self.data_dir,
            validation_split=0.2,
            subset="training",
            seed=123,
            image_size=(self.img_height, self.img_width),
            batch_size=self.batch_size)

        val_ds = tf.keras.preprocessing.image_dataset_from_directory(
            self.data_dir,
            validation_split=0.2,
            subset="validation",
            seed=123,
            image_size=(self.img_height, self.img_width),
            batch_size=self.batch_size)

        self.class_names = train_ds.class_names
        self.train_ds = train_ds
        self.val_ds = val_ds

    def data_augmentation_fun(self):
        data_augmentation = keras.Sequential(
            [
                layers.experimental.preprocessing.RandomFlip("horizontal",
                                                             input_shape=(self.img_height,
                                                                          self.img_width,
                                                                          3)),
                layers.experimental.preprocessing.RandomRotation(0.1),
                layers.experimental.preprocessing.RandomZoom(0.1),
            ]
        )
        self.data_augmentation = data_augmentation

    def model_create(self):
        self.data_augmentation_fun()
        model = Sequential([
            self.data_augmentation,
            layers.experimental.preprocessing.Rescaling(1. / 255, input_shape=(self.img_height,
                                                                               self.img_width, 3)),
            layers.Conv2D(16, 3, padding='same', activation='relu'),
            layers.MaxPooling2D(),
            layers.Conv2D(32, 3, padding='same', activation='relu'),
            layers.MaxPooling2D(),
            layers.Conv2D(64, 3, padding='same', activation='relu'),
            layers.MaxPooling2D(),
            layers.Flatten(),
            layers.Dense(128, activation='relu'),
            layers.Dense(self.num_classes)
        ])

        model.compile(optimizer='adam',
                      loss=tf.keras.losses.SparseCategoricalCrossentropy(from_logits=True),
                      metrics=['accuracy'])
        return model

    def train(self, model):
        history = model.fit(
            self.train_ds,
            validation_data=self.val_ds,
            epochs=self.epochs
        )
        if not os.path.exists(self.model_path):
            os.makedirs(self.model_path)

        model.save(f'{self.model_path}/model.h5')
        return history

    def visualize_results(self, history):
        acc = history.history['accuracy']
        val_acc = history.history['val_accuracy']

        loss = history.history['loss']
        val_loss = history.history['val_loss']

        epochs_range = range(self.epochs)

        plt.figure(figsize=(8, 8))
        plt.subplot(1, 2, 1)
        plt.plot(epochs_range, acc, label='Training Accuracy')
        plt.plot(epochs_range, val_acc, label='Validation Accuracy')
        plt.legend(loc='lower right')
        plt.title('Training and Validation Accuracy')

        plt.subplot(1, 2, 2)
        plt.plot(epochs_range, loss, label='Training Loss')
        plt.plot(epochs_range, val_loss, label='Validation Loss')
        plt.legend(loc='upper right')
        plt.title('Training and Validation Loss')
        plt.show()

    def start(self):
        self.dataloader()
        print("Dataset Dir: ", self.data_dir)
        print("Number of class: ", self.num_classes)
        print("Total Images: ", self.image_count)
        print("batch_size: ", self.batch_size)
        print("img_width: ", self.img_width)
        print("img_height: ", self.img_height)
        print("class_names: ", self.class_names)
        print("Model Directory: ", self.model_path + '/model.h5')
        print("================================================")
        print("================= Starting Training =================")
        model = self.model_create()
        model.summary()
        history = self.train(model)

        self.visualize_results(history)


if __name__ == '__main__':
    obj = Classifier()
    obj.start()



import auto_face_recognition
import pathlib
import glob
import os

obj = auto_face_recognition.AutoFaceRecognition()


class DatasetCreate:
    def __init__(self, dataset_path='datasets', class_name='Demo',
                 haarcascade_path='haarcascade/haarcascade_frontalface_default.xml',
                 eyecascade_path='haarcascade/haarcascade_eye.xml', eye_detect=False,
                 save_face_only=True, no_of_samples=100,
                 width=128, height=128, color_mode=False):
        self.dataset_path = dataset_path
        self.class_name = class_name
        self.haarcascade_path = haarcascade_path
        self.eyecascade_path = eyecascade_path
        self.eye_detect = eye_detect
        self.save_face_only = save_face_only
        self.no_of_samples = no_of_samples
        self.width = width
        self.height = height
        self.color_mode = color_mode

    def datasetcreate(self):
        obj.datasetcreate(dataset_path=self.dataset_path, class_name=self.class_name,
                          haarcascade_path=self.haarcascade_path,
                          eyecascade_path=self.eyecascade_path, eye_detect=self.eye_detect,
                          save_face_only=self.save_face_only, no_of_samples=self.no_of_samples,
                          width=self.width, height=self.height, color_mode=self.color_mode)


class FaceRecognizerTrain:
    def __init__(self, data_dir='datasets', batch_size=32, img_height=128, img_width=128, epochs=10,
                 model_path='model', pretrained=None, base_model_trainable=False):
        self.data_dir = data_dir
        self.batch_size = batch_size
        self.img_height = img_height
        self.img_width = img_width
        self.class_names = None
        self.epochs = epochs
        self.model_path = model_path
        self.pretrained = pretrained
        self.base_model_trainable = base_model_trainable

    def train(self):
        obj.face_recognition_train(data_dir=self.data_dir, batch_size=self.batch_size, img_height=self.img_height,
                                   img_width=self.img_width, epochs=self.epochs, model_path=self.model_path,
                                   pretrained=self.pretrained, base_model_trainable=self.base_model_trainable)


class Predict:
    def __init__(self, class_name, img_height=128, img_width=128,
                 haarcascade_path='haarcascade/haarcascade_frontalface_default.xml',
                 eyecascade_path='haarcascade/haarcascade_eye.xml', model_path='model',
                 color_mode=False):
        self.img_height = img_height
        self.img_width = img_width
        self.img_src_preprocessed = None
        self.class_name = class_name
        self.haarcascade_path = haarcascade_path
        self.eyecascade_path = eyecascade_path
        self.color_mode = color_mode
        self.model_path = model_path

    def predictfaces(self):
        obj.predict_faces(class_name=self.class_name, img_height=self.img_height, img_width=self.img_width,
                          haarcascade_path=self.haarcascade_path,
                          eyecascade_path=self.eyecascade_path, model_path=self.model_path,
                          color_mode=self.color_mode)



import webbrowser


def google_photos():
    try:
        url = "https://photos.google.com/"
        webbrowser.open(url)
        return True
    except Exception as e:
        print(e)
        return False



import setuptools
from setuptools import find_namespace_packages


with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="JarvisAI",
    version="3.1",
    author="Dipesh",
    author_email="dipeshpal17@gmail.com",
    description="JarvisAI is AI python library",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Dipeshpal/Jarvis_AI",
    include_package_data=True,
    packages=find_namespace_packages(include=['JarvisAI.*', 'JarvisAI']),
    install_requires=['numpy', 'gtts==2.2.1', 'playsound==1.2.2',
                      'SpeechRecognition==3.8.1', 'pipwin==0.5.0', 'lxml==4.6.1', 'pyjokes',
                      'beautifulsoup4==4.9.3', 'wikipedia==1.4.0', 'auto_face_recognition', 'transformers==4.3.2',
                      'lazyme==0.0.23', 'librosa==0.8.0', "torch==1.7.1", "requests", "opencv-contrib-python==4.5.2.52",
                      "opencv-python==4.5.2.52", "cvzone==1.1.1"],

    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)



import subprocess
import configparser


def show_me_my_images():
    """
    This method will open image directory
    :return: Bool
    """
    config = configparser.ConfigParser()
    config.read('config/config.ini')
    try:
        path = config['default']['photos']
        path = path.split(",")
        for i in path:
            subprocess.Popen(f'explorer "{i}"')
        return True
    except Exception as e:
        print(f"Unable to locate default directory: {e}")
        return False



import requests
import webbrowser


class JarvisAIAPI:
    def __init__(self):
        pass

    def get_user_data(self, token):
        # https://jarvis-ai-api.herokuapp.com/get_user_data_by_token_api/?token=5ec64be7ff718ac25917c198f3d7a4
        url = f'https://jarvis-ai-api.herokuapp.com/get_user_data_by_token_api/?token={token}'
        x = requests.get(url)
        return x.status_code, x.json()

    def set_user_data(self):
        url = 'https://jarvis-ai-api.herokuapp.com/login/'
        webbrowser.open(url, new=2)


if __name__ == '__main__':
    obj = JarvisAIAPI()
    status, res = obj.get_user_data("xxxxx")
    print(status)
    print(res)
    print(type(res))
